<?php
namespace Auction\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A question concerning an item on auction..
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="auction_question")
 */
class Question
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="Auction")
	 * @ORM\JoinColumn(nullable=false, name="auction_id")
	 **/
	protected $auction;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="profile_id")
	 **/
	protected $profile;

	/**
	 * @ORM\ManyToOne(targetEntity="Reply")
	 * @ORM\JoinColumn(nullable=true, name="auction_reply_id")
	 **/
	protected $reply;


	/* ------------------------------------ Question ------------------------------------ */
	/**
	 * @ORM\Column(type="text")
	 */
	protected $question = '';


	/* ------------------------------------ Tracking ------------------------------------ */
	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		is_null($this->company)
			&& $this->company = \Utility\Registry::resolveCompanyContext($this->company);
		is_null($this->profile)
			&& $this->profile = \Utility\Registry::resolveProfileContext($this->profile);
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['auction']))
			&& $data['auction'] = (in_array('auction', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->auction)
				? (!$showIdentifiers ? $this->auction->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->auction->id)
				: null;
		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['profile']))
			&& $data['profile'] = (in_array('profile', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->profile)
				? (!$showIdentifiers ? $this->profile->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->profile->id)
				: null;
		($includeAll || isset($intersect['reply']))
			&& $data['reply'] = (in_array('reply', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->reply)
				? (!$showIdentifiers ? $this->reply->toArray(
						array(), array(), true, false
						/* $expand, $intersect, $showIdentifiers, ($expandAll - 1) */
						) : $this->reply->id)
				: null;
		($includeAll || isset($intersect['question']))
			&& $data['question'] = $this->question;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['auction'])
			&& $this->auction = $data['auction'];
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['profile'])
			&& $this->profile = $data['profile'];
		isset($data['reply'])
			&& $this->reply = $data['reply'];
		isset($data['question'])
			&& $this->question = $data['question'];
	}



}

